@@ -280,39 +280,36 @@ JareckiLysyanskayaRVSS::JareckiLysyanskayaRVSS
 	(const size_t n_in, const size_t t_in,
 	mpz_srcptr p_CRS, mpz_srcptr q_CRS, mpz_srcptr g_CRS, mpz_srcptr h_CRS,
 	const unsigned long int fieldsize, const unsigned long int subgroupsize):
-			F_size(fieldsize), G_size(subgroupsize), n(n_in), t(t_in)
+			F_size(fieldsize), G_size(subgroupsize),
+			n(n_in), t(t_in)
 {
-	mpz_init_set(p, p_CRS), mpz_init_set(q, q_CRS), mpz_init_set(g, g_CRS),
-		mpz_init_set(h, h_CRS);
+	mpz_init_set(p, p_CRS), mpz_init_set(q, q_CRS), mpz_init_set(g, g_CRS),	mpz_init_set(h, h_CRS);
 
 	mpz_init_set_ui(a_i, 0L), mpz_init_set_ui(hata_i, 0L);
 	mpz_init_set_ui(alpha_i, 0L), mpz_init_set_ui(hatalpha_i, 0L);
-	for (size_t j = 0; j < n_in; j++)
+	alpha_ij.resize(n);
+	hatalpha_ij.resize(n);
+	C_ik.resize(n);
+	for (size_t i = 0; i < n; i++)
 	{
-		std::vector<mpz_ptr> *vtmp1 = new std::vector<mpz_ptr>;
-		for (size_t i = 0; i < n_in; i++)
+		for (size_t j = 0; j < n; j++)
 		{
 			mpz_ptr tmp1 = new mpz_t();
 			mpz_init(tmp1);
-			vtmp1->push_back(tmp1);
+			alpha_ij[i].push_back(tmp1);
 		}
-		alpha_ij.push_back(*vtmp1);
-		std::vector<mpz_ptr> *vtmp2 = new std::vector<mpz_ptr>;
-		for (size_t i = 0; i < n_in; i++)
+		for (size_t j = 0; j < n; j++)
 		{
 			mpz_ptr tmp2 = new mpz_t();
 			mpz_init(tmp2);
-			vtmp2->push_back(tmp2);
+			hatalpha_ij[i].push_back(tmp2);
 		}
-		hatalpha_ij.push_back(*vtmp2);
-		std::vector<mpz_ptr> *vtmp3 = new std::vector<mpz_ptr>;
-		for (size_t k = 0; k <= t_in; k++)
+		for (size_t k = 0; k <= t; k++)
 		{
 			mpz_ptr tmp3 = new mpz_t();
 			mpz_init(tmp3);
- 			vtmp3->push_back(tmp3);
+ 			C_ik[i].push_back(tmp3);
 		}
-		C_ik.push_back(*vtmp3);
 	}
 
 	// Do the precomputation for the fast exponentiation.
@@ -334,6 +331,8 @@ bool JareckiLysyanskayaRVSS::CheckGroup
 		// Compute $k := (p - 1) / q$
 		mpz_set(k, p);
 		mpz_sub_ui(k, k, 1L);
+		if (!mpz_cmp_ui(q, 0L))
+			throw false;
 		mpz_div(k, k, q);
 
 		// Check whether $p$ and $q$ have appropriate sizes.
@@ -386,6 +385,32 @@ bool JareckiLysyanskayaRVSS::CheckGroup
 	}
 }
 
+bool JareckiLysyanskayaRVSS::CheckElement
+	(mpz_srcptr a) const
+{
+	mpz_t foo;
+	mpz_init(foo);
+
+	try
+	{
+		// Check whether $0 < a < p$.
+		if ((mpz_cmp_ui(a, 0L) <= 0) || (mpz_cmp(a, p) >= 0))
+			throw false;
+		
+		// Check whether $a^q \equiv 1 \pmod{p}$.
+		mpz_powm(foo, a, q, p);
+		if (mpz_cmp_ui(foo, 1L))
+			throw false;
+		
+		throw true;
+	}
+	catch (bool return_value)
+	{
+		mpz_clear(foo);
+		return return_value;
+	}
+}
+
 bool JareckiLysyanskayaRVSS::Share
 	(const size_t i, aiounicast *aiou, CachinKursawePetzoldShoupRBC *rbc,
 	std::ostream &err, const bool simulate_faulty_behaviour)
@@ -453,6 +478,12 @@ bool JareckiLysyanskayaRVSS::Share
 						complaints.push_back(j);
 						break;
 					}
+					if (!CheckElement(C_ik[j][k]))
+					{
+						err << "P_" << i << ": bad C_ik received; complaint against P_" << j << std::endl;
+						complaints.push_back(j);
+						mpz_set_ui(C_ik[j][k], 0L); // indicates an error
+					}
 				}
 			}
 		}
@@ -481,13 +512,13 @@ bool JareckiLysyanskayaRVSS::Share
 				{
 					mpz_add_ui(alpha_ij[i][j], alpha_ij[i][j], 1L);
 				}
-				if (!aiou->Send(alpha_ij[i][j], j))
+				if (!aiou->Send(alpha_ij[i][j], j, aiou->aio_timeout_very_short))
 				{
 					err << "P_" << i << ": sending alpha_ij failed; complaint against P_" << j << std::endl;
 					complaints.push_back(j);
 					continue;
 				}
-				if (!aiou->Send(hatalpha_ij[i][j], j))
+				if (!aiou->Send(hatalpha_ij[i][j], j, aiou->aio_timeout_very_short))
 				{
 					err << "P_" << i << ": sending hatalpha_ij failed; complaint against P_" << j << std::endl;
 					complaints.push_back(j);
@@ -511,12 +542,24 @@ bool JareckiLysyanskayaRVSS::Share
 					complaints.push_back(j);
 					continue;
 				}
+				if (mpz_cmpabs(alpha_ij[j][i], q) >= 0)
+				{
+					err << "P_" << i << ": bad alpha_ij received; complaint against P_" << j << std::endl;
+					complaints.push_back(j);
+					mpz_set_ui(alpha_ij[j][i], 0L); // indicates an error
+				}
 				if (!aiou->Receive(hatalpha_ij[j][i], j, aiou->aio_scheduler_direct))
 				{
 					err << "P_" << i << ": receiving hatalpha_ij failed; complaint against P_" << j << std::endl;
 					complaints.push_back(j);
 					continue;
 				}
+				if (mpz_cmpabs(hatalpha_ij[j][i], q) >= 0)
+				{
+					err << "P_" << i << ": bad hatalpha_ij received; complaint against P_" << j << std::endl;
+					complaints.push_back(j);
+					mpz_set_ui(hatalpha_ij[j][i], 0L); // indicates an error
+				}
 			}
 		}
 		for (size_t j = 0; j < n; j++)
@@ -553,9 +596,12 @@ bool JareckiLysyanskayaRVSS::Share
 		}
 		mpz_set_ui(rhs, n); // broadcast end marker
 		rbc->Broadcast(rhs);
-		complaints.clear(), complaints_from.clear(); // reset
+		complaints_counter.clear(), complaints_from.clear(); // reset for final complaint resolution
 		for (size_t j = 0; j < n; j++)
 			complaints_counter.push_back(0); // initialize counter
+		for (std::vector<size_t>::iterator it = complaints.begin(); it != complaints.end(); ++it)
+			complaints_counter[*it]++; // count my own complaints
+		complaints.clear();
 		for (size_t j = 0; j < n; j++)
 		{
 			if (j != i)
@@ -614,14 +660,13 @@ bool JareckiLysyanskayaRVSS::Share
 		rbc->Broadcast(lhs);
 		for (size_t j = 0; j < n; j++)
 		{
+			if (complaints_counter[j] > t)
+			{
+				complaints.push_back(j);
+				continue;
+			}
 			if (j != i)
 			{
-				size_t who;
-				if (complaints_counter[j] > t)
-				{
-					complaints.push_back(j);
-					continue;
-				}
 				size_t cnt = 0;
 				do
 				{
@@ -631,7 +676,7 @@ bool JareckiLysyanskayaRVSS::Share
 						complaints.push_back(j);
 						break;
 					}
-					who = mpz_get_ui(lhs);
+					size_t who = mpz_get_ui(lhs);
 					if (who >= n)
 						break; // end marker received
 					if (!rbc->DeliverFrom(foo, j))
@@ -640,12 +685,24 @@ bool JareckiLysyanskayaRVSS::Share
 						complaints.push_back(j);
 						break;
 					}
+					if (mpz_cmpabs(foo, q) >= 0)
+					{
+						err << "P_" << i << ": bad foo received; complaint against P_" << j << std::endl;
+						complaints.push_back(j);
+						mpz_set_ui(foo, 0L); // indicates an error
+					}
 					if (!rbc->DeliverFrom(bar, j))
 					{
 						err << "P_" << i << ": receiving bar failed; complaint against P_" << j << std::endl;
 						complaints.push_back(j);
 						break;
 					}
+					if (mpz_cmpabs(bar, q) >= 0)
+					{
+						err << "P_" << i << ": bad bar received; complaint against P_" << j << std::endl;
+						complaints.push_back(j);
+						mpz_set_ui(bar, 0L); // indicates an error
+					}
 					mpz_t alpha, hatalpha;
 					mpz_init_set(alpha, foo), mpz_init_set(hatalpha, bar);
 					// compute LHS for the check
@@ -777,6 +834,11 @@ bool JareckiLysyanskayaRVSS::Share_twoparty
 					err << "P_" << i << ": receiving C_ik failed" << std::endl;
 					throw false;
 				}
+				if (!CheckElement(C_ik[j][0]))
+				{
+					err << "P_" << i << ": bad C_ik received" << std::endl;
+					throw false;
+				}
 			}
 		}
 		// Set $F_{a_i}(z) = \prod_{k=0}^t (C_{ik})^{z^k}$.
@@ -818,10 +880,19 @@ bool JareckiLysyanskayaRVSS::Reconstruct
 	// initialize
 	mpz_t foo, bar, lhs, rhs;
 	mpz_init(foo), mpz_init(bar), mpz_init(lhs), mpz_init(rhs);
+	std::vector<mpz_ptr> shares;
+	for (size_t j = 0; j < n; j++)
+	{
+		mpz_ptr tmp1 = new mpz_t();
+		mpz_init(tmp1);
+		shares.push_back(tmp1);
+	}
 
 	// set ID for RBC
 	std::stringstream myID;
 	myID << "JareckiLysyanskayaRVSS::Reconstruct()" << p << q << g << h << n << t;
+	for (std::vector<size_t>::const_iterator it = complaints.begin(); it != complaints.end(); ++it)
+		myID << "[" << *it << "]";
 	rbc->setID(myID.str());
 
 	try
@@ -834,38 +905,53 @@ bool JareckiLysyanskayaRVSS::Reconstruct
 		}
 		for (std::vector<size_t>::const_iterator it = complaints.begin(); it != complaints.end(); ++it)
 		{
-			// broadcast shares for reconstruction of $z_i$ (where $i = *it$) 
-			rbc->Broadcast(alpha_ij[*it][i]);
-			rbc->Broadcast(hatalpha_ij[*it][i]);
+			if (std::find(Qual.begin(), Qual.end(), *it) == Qual.end())
+			{
+				err << "P_" << i << ": reconstruction of z_i failed because P_" << *it << " not in Qual" << std::endl;
+				throw false;
+			}
 			// prepare for collecting shares
 			std::vector<size_t> parties;
-			parties.push_back(i); // shares of $P_i$ are always available
+			parties.push_back(i); // share of this player is always available
+			mpz_set(shares[i], alpha_ij[*it][i]);
+			// broadcast shares for reconstruction of $z_i$ (where $i = *it$ is here the index of the failed party)
+			if ((std::find(complaints.begin(), complaints.end(), i) == complaints.end()) && (std::find(Qual.begin(), Qual.end(), i) != Qual.end()))
+			{
+				rbc->Broadcast(alpha_ij[*it][i]);
+				rbc->Broadcast(hatalpha_ij[*it][i]);
+			}
 			// collect shares $\alpha_{ij}$ and $\hat{\alpha}_{ij}$ of other parties from Qual
 			for (std::vector<size_t>::iterator jt = Qual.begin(); jt != Qual.end(); ++jt)
 			{
 				if ((*jt != i) && (std::find(complaints.begin(), complaints.end(), *jt) == complaints.end()))
 				{
-					if (rbc->DeliverFrom(alpha_ij[*it][*jt], *jt) && rbc->DeliverFrom(hatalpha_ij[*it][*jt], *jt))
+					if (rbc->DeliverFrom(foo, *jt) && rbc->DeliverFrom(bar, *jt))
 					{
-						// compute LHS for the check
-						mpz_fpowm(fpowm_table_g, foo, g, alpha_ij[*it][*jt], p);
-						mpz_fpowm(fpowm_table_h, bar, h, hatalpha_ij[*it][*jt], p);
-						mpz_mul(lhs, foo, bar);
-						mpz_mod(lhs, lhs, p);
-						// compute RHS for the check
-						mpz_set_ui(rhs, 1L);
-						for (size_t k = 0; k <= t; k++)
-						{
-							mpz_ui_pow_ui(foo, *jt + 1, k); // adjust index $j$ in computation
-							mpz_powm(bar, C_ik[*it][k], foo, p);
-							mpz_mul(rhs, rhs, bar);
-							mpz_mod(rhs, rhs, p);
-						}
-						// check equation (4)
-						if (mpz_cmp(lhs, rhs))
+						if ((mpz_cmpabs(foo, q) >= 0) || (mpz_cmpabs(bar, q) >= 0))
 							err << "P_" << i << ": bad share received from " << *jt << std::endl;
 						else
-							parties.push_back(*jt);
+						{
+							mpz_set(shares[*jt], foo); // save the received share for later following interpolation
+							// compute LHS for the check
+							mpz_fpowm(fpowm_table_g, foo, g, foo, p);
+							mpz_fpowm(fpowm_table_h, bar, h, bar, p);
+							mpz_mul(lhs, foo, bar);
+							mpz_mod(lhs, lhs, p);
+							// compute RHS for the check
+							mpz_set_ui(rhs, 1L);
+							for (size_t k = 0; k <= t; k++)
+							{
+								mpz_ui_pow_ui(foo, *jt + 1, k); // adjust index $j$ in computation
+								mpz_powm(bar, C_ik[*it][k], foo, p);
+								mpz_mul(rhs, rhs, bar);
+								mpz_mod(rhs, rhs, p);
+							}
+							// check equation (4)
+							if (mpz_cmp(lhs, rhs))
+								err << "P_" << i << ": bad share received from " << *jt << std::endl;
+							else
+								parties.push_back(*jt);
+						}
 					}
 					else
 						err << "P_" << i << ": no share received from " << *jt << std::endl;					
@@ -910,7 +996,7 @@ bool JareckiLysyanskayaRVSS::Reconstruct
 				}
 				mpz_mul(rhs, rhs, lhs);
 				mpz_mod(rhs, rhs, q);
-				mpz_mul(bar, alpha_ij[*it][*jt], rhs); // use the provided shares (interpolation points)
+				mpz_mul(bar, rhs, shares[*jt]); // use the provided shares (interpolation points)
 				mpz_mod(bar, bar, q);
 				mpz_add(foo, foo, bar);
 				mpz_mod(foo, foo, q);
@@ -926,6 +1012,12 @@ bool JareckiLysyanskayaRVSS::Reconstruct
 		// unset ID for RBC
 		rbc->unsetID();
 		// release
+		for (size_t j = 0; j < shares.size(); j++)
+		{
+			mpz_clear(shares[j]);
+			delete [] shares[j];
+		}
+		shares.clear();
 		mpz_clear(foo), mpz_clear(bar), mpz_clear(lhs), mpz_clear(rhs);
 		// return
 		return return_value;
@@ -982,12 +1074,10 @@ JareckiLysyanskayaEDCF::JareckiLysyanskayaEDCF
 	const unsigned long int fieldsize, const unsigned long int subgroupsize):
 			F_size(fieldsize), G_size(subgroupsize), n(n_in), t(t_in)
 {
-	mpz_init_set(p, p_CRS), mpz_init_set(q, q_CRS), mpz_init_set(g, g_CRS),
-		mpz_init_set(h, h_CRS);
+	mpz_init_set(p, p_CRS), mpz_init_set(q, q_CRS), mpz_init_set(g, g_CRS),	mpz_init_set(h, h_CRS);
 
 	// initialize RVSS
-	rvss = new JareckiLysyanskayaRVSS(n_in, t_in,
-		p_CRS, q_CRS, g_CRS, h_CRS, fieldsize, subgroupsize);	
+	rvss = new JareckiLysyanskayaRVSS(n_in, t_in, p_CRS, q_CRS, g_CRS, h_CRS, fieldsize, subgroupsize);	
 
 	// Do the precomputation for the fast exponentiation.
 	fpowm_table_g = new mpz_t[TMCG_MAX_FPOWM_T]();
@@ -1020,7 +1110,7 @@ bool JareckiLysyanskayaEDCF::Flip
 	// initialize
 	mpz_t foo, bar, lhs, rhs;
 	std::vector<mpz_ptr> a_i, hata_i;
-	std::vector<size_t> complaints, complaints_counter;
+	std::vector<size_t> complaints;
 	mpz_init(foo), mpz_init(bar), mpz_init(lhs), mpz_init(rhs);
 	for (size_t j = 0; j < n; j++)
 	{
@@ -1066,12 +1156,24 @@ bool JareckiLysyanskayaEDCF::Flip
 					complaints.push_back(j);
 					continue;
 				}
+				if (mpz_cmpabs(a_i[j], q) >= 0)
+				{
+					err << "P_" << i << ": bad a_i received; complaint against P_" << j << std::endl;
+					complaints.push_back(j);
+					mpz_set_ui(a_i[j], 0L); // indicates an error
+				}
 				if (!rbc->DeliverFrom(hata_i[j], j))
 				{
 					err << "P_" << i << ": receiving hata_i failed; complaint against P_" << j << std::endl;
 					complaints.push_back(j);
 					continue;
 				}
+				if (mpz_cmpabs(hata_i[j], q) >= 0)
+				{
+					err << "P_" << i << ": bad hata_i received; complaint against P_" << j << std::endl;
+					complaints.push_back(j);
+					mpz_set_ui(hata_i[j], 0L); // indicates an error
+				}
 			}
 		}
 		for (size_t j = 0; j < n; j++)
@@ -1185,7 +1287,27 @@ bool JareckiLysyanskayaEDCF::Flip_twoparty
 			if (j != i)
 			{
 				in >> a_i[j];
+				if (!in.good())
+				{
+					err << "P_" << i << ": receiving a_i failed" << std::endl;
+					throw false;
+				}
+				if (mpz_cmpabs(a_i[j], q) >= 0)
+				{
+					err << "P_" << i << ": bad a_i received" << std::endl;
+					throw false;
+				}
 				in >> hata_i[j];
+				if (!in.good())
+				{
+					err << "P_" << i << ": receiving hata_i failed" << std::endl;
+					throw false;
+				}
+				if (mpz_cmpabs(hata_i[j], q) >= 0)
+				{
+					err << "P_" << i << ": bad hata_i received" << std::endl;
+					throw false;
+				}
 				// compute LHS for the check
 				mpz_fspowm(fpowm_table_g, foo, g, a_i[j], p);
 				mpz_fspowm(fpowm_table_h, bar, h, hata_i[j], p);
