@@ -1,7 +1,7 @@
 /*******************************************************************************
    This file is part of LibTMCG.
 
- Copyright (C) 2004, 2005  Heiko Stamer <stamer@gaos.org>
+ Copyright (C) 2004, 2005, 2016, 2017  Heiko Stamer <HeikoStamer@gmx.net>
 
    LibTMCG is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -21,15 +21,10 @@
 #ifndef INCLUDED_mpz_spowm_H
 	#define INCLUDED_mpz_spowm_H
 	
-	// config.h
-	#ifdef HAVE_CONFIG_H
-		#include "config.h"
-	#endif
-	
 	#include <stdio.h>
 	#include <assert.h>
 	
-	// GNU multiple precision library
+	/* GNU multiple precision library */
 	#include <gmp.h>
 	
 	#include "mpz_srandom.h"
@@ -38,7 +33,9 @@
 		extern "C"
 		{
 	#endif
-			/* Kocher's efficient blinding technique for modexp */
+			/* Kocher's efficient blinding technique for modular exponentiation [Ko96] 
+                           -- cf. scientific discussion e.g. https://eprint.iacr.org/2013/447 
+			   https://eprint.iacr.org/2014/869 and https://eprint.iacr.org/2016/597 */
 			void mpz_spowm_init
 				(mpz_srcptr x, mpz_srcptr p);
 			
@@ -48,7 +45,14 @@
 			void mpz_spowm_clear
 				();
 			
-			/* Chaum's blinding technique for modular exponentiation */
+			/* Chaum's blinding technique for modular exponentiation
+			   -- cf. scientific discussion e.g. https://eprint.iacr.org/2013/447 
+			   https://eprint.iacr.org/2014/869 and https://eprint.iacr.org/2016/597 */
+			void mpz_spowm_baseblind
+				(mpz_ptr res, mpz_srcptr m, mpz_srcptr x, mpz_srcptr p);
+
+			/* Use the constant-time function mpz_powm_sec() from libgmp, if available. 
+   			   Otherwise Chaum's blinding technique for modular exponentiation is applied. */
 			void mpz_spowm
 				(mpz_ptr res, mpz_srcptr m, mpz_srcptr x, mpz_srcptr p);
 			
@@ -58,7 +62,7 @@
 			
 			void mpz_fpowm_precompute
 				(mpz_t fpowm_table[],
-				mpz_srcptr m, mpz_srcptr p, size_t t);
+				mpz_srcptr m, mpz_srcptr p, const size_t t);
 			
 			void mpz_fpowm
 				(mpz_t fpowm_table[],
@@ -66,7 +70,7 @@
 			
 			void mpz_fpowm_ui
 				(mpz_t fpowm_table[],
-				mpz_ptr res, mpz_srcptr m, unsigned long int x_ui, mpz_srcptr p);
+				mpz_ptr res, mpz_srcptr m, const unsigned long int x_ui, mpz_srcptr p);
 			
 			void mpz_fspowm
 				(mpz_t fpowm_table[],
